@@ -68,9 +68,9 @@ extern const fenv_t	npy__fe_dfl_env;
 #define	__fldcw(__cw)		__asm __volatile("fldcw %0" : : "m" (__cw))
 #define	__fldenv(__env)		__asm __volatile("fldenv %0" : : "m" (__env))
 #define	__fnclex()		__asm __volatile("fnclex")
-#define	__fnstenv(__env)	__asm("fnstenv %0" : "=m" (*(__env)))
-#define	__fnstcw(__cw)		__asm("fnstcw %0" : "=m" (*(__cw)))
-#define	__fnstsw(__sw)		__asm("fnstsw %0" : "=am" (*(__sw)))
+#define	__fnstenv(__env)	__asm __volatile("fnstenv %0" : "=m" (*(__env)))
+#define	__fnstcw(__cw)		__asm __volatile("fnstcw %0" : "=m" (*(__cw)))
+#define	__fnstsw(__sw)		__asm __volatile("fnstsw %0" : "=am" (*(__sw)))
 #define	__fwait()		__asm __volatile("fwait")
 
 static __inline int
